.outer {
  display: flex;
  flex-direction: column;
  height: 100vh;
  overflow: hidden;
}

.contentContainer {
  flex: 1;
  display: flex;
  flex-direction: row;
  overflow: hidden;
}

.sidebar {
  width: 320px;
  box-shadow: inset -1px 0 0 0 var(--color-keyline-default);
  overflow-y: auto;
}

.main {
  flex: 1;
  overflow-y: auto;
}